#include <VRS51L3074_SDCC.h>

/* LCD control signals */
#define	LCD_RD	0
#define	LCD_A0	3
#define	LCD_WR	1
#define	LCD_CS	4


void LCD_Write(unsigned char dat)
{
/* S1D13305: 0ns address setup, 120ns data setup, 120ns strobe width */
  
        P0=dat;	        	// write data
	P0PINCFG=0;		// P0 = Outputs, put data in bus
	P2&=~((1<<LCD_WR)|(1<<LCD_CS));	// Lower WR+CS, the ANL used is 2 cycles, so it will add a 50ns delay @40MHz
	_asm
	  DA A              ; 100ns delay, 4 cycles for a 40Mhz processor
        _endasm;
	P2|=((1<<LCD_WR)|(1<<LCD_CS));	// Rise WR+CS, the ANL used is 2 cycles, so it will add a 50ns delay @40MHz
	_asm
	  DA A
	  DA A              ; 400ns delay, 16 cycles for a 40Mhz processor
	  DA A              ; RET will add more delay
	  DA A
        _endasm;
}



void LCD_WriteCmd(unsigned char cmd)
{
	P2|=(1<<LCD_A0);	// Rise A0 (Cmd)
	LCD_Write(cmd);
	P2&=~(1<<LCD_A0);	// Lower A0 (Data)
}

void LCD_WriteStrCmd(unsigned char *cmd,unsigned char len)
{
	LCD_WriteCmd(*cmd++);
	while(--len) {
		LCD_Write(*cmd++);
	}
}


unsigned char LCD_Read()
{
/* S1D13305: 0ns address setup, 50ns data delay, 120ns strobe width */

unsigned char dat;

	P0PINCFG=0xFF;		// P0 = Inputs, remove data from bus, the MOV used is 2 cycles, so it will add a 50ns delay @40MHz
	P2&=~((1<<LCD_RD)|(1<<LCD_CS));	// Lower RD+CS, the ANL used is 2 cycles, so it will add a 50ns delay @40MHz
	_asm
	  DA A              ; 100ns delay, 4 cycles for a 40Mhz processor
        _endasm;
	dat=P0;	                // read data, the MOV used is 3 cycles, so it will add a 75ns delay @40MHz
	P2|=((1<<LCD_RD)|(1<<LCD_CS));	// Rise RD+CS, the ANL used is 2 cycles, so it will add a 50ns delay @40MHz
	_asm
	  DA A
	  DA A              ; 400ns delay, 16 cycles for a 40Mhz processor
	  DA A              ; return will add more delay
	  DA A
        _endasm;
	return(dat);
}

unsigned char LCD_ReadData()
{

unsigned char dat;

	P2|=(1<<LCD_A0);	// Rise A0 (Read Data)
	dat=LCD_Read();
	P2&=~(1<<LCD_A0);	// Lower A0
	return(dat);
}


void LCD_init ()
{
__code const static unsigned char init_string1[]={
0x40,				// INIT
0x30,	                 // 32 char CGRAM, 8 pixel chars, no top-screen corr, LCD, normal
0x87,0x7,		// 8x8 characters
39,59,			// 320 pixels, 8 pixel characters => 40 characters per line (43)
239,				// 240 lines (30 txt lines)
40,0				// virtual screen = display screen
};

__code const static unsigned char init_string2[]={
0x44,				// SCROLL
0, 				// SAD1L: start address screen 1 (0x0000)
0, 				// SAD1H: (for text screen)
239, 				// SL1  : number of scrolling lines screen 1 (239 dec.)
0xB0, 			// SAD2L: start address screen 2 (1200=0x4b0)
0x04, 			// SAD2H: (for the graphics screen)
239, 				// SL2  : number of scrolling lines screen 2
0, 				// SAD3L:
0, 				// SAD3H:
0, 				// SAD4L:
0	 				// SAD4H:
};

__code const static unsigned char init_string3[]={
0x5D,				// CSRFORM	 
0x04,				// CRX: set the horizontal size of the cursor (4 pixel)
0x86				// CRY: set the vertical size of the cursor (6 pixel) CM : set the cursor display mode to block cursor
};

		
__code const static unsigned char init_string4[]={
0x5A,				// HDOT_SCR (0x5a), Sets horizontal scroll position (scroll rate).
0					// 1 pixel scroll
};

__code const static unsigned char init_string5[]={
0x5B,				// OVLAY, Selects layered screen composition and screen text/graphics mode.
0x01      	//MX0, MX1: set the text/graphics overlay mode to EXOR
//		Bit 2-3 : DM1, DM2: set the display mode of screen block 1 and 3
//		Bit 4   : OV: set two or three layer compsition in gfx-mode
//		Bit 5-7 :  0: not used (set to 0)
};

__code const static unsigned char init_string6[]={
0x59,				// send command DISP_ON (0x59)
0x16                  	// cursor flash rate = 2 Hz, SAD1/2 no flashing, SAD3 off
};

	P0=0;
        P2|=0x1B;		// Make sure Outputs are High
        P2PINCFG&=0xE4;        // xxxx0000 P2.0,1,2,3 = output
	// MsDelay ( 1000 );				// wait for LCD to reset itself

	LCD_WriteStrCmd ( init_string1,sizeof(init_string1) );
	LCD_WriteStrCmd ( init_string2,sizeof(init_string2) );
	LCD_WriteStrCmd ( init_string3,sizeof(init_string3) );
	LCD_WriteCmd ( 0x4C );		// CSRDIR_RIGHT, sets the direction of automatic cursor increment to right.
	LCD_WriteStrCmd ( init_string4,sizeof(init_string4) );
	LCD_WriteStrCmd ( init_string5,sizeof(init_string5) );
	LCD_WriteStrCmd ( init_string6,sizeof(init_string6) );
}


